/*
stl_ptr's are auto pointers with no ownership so they can be copied
without causing algorithms to fail.  They use a primitive kind of
reference counting  The pointer is deleted when the last stl_ptr
pointing to it goes out of scope.
*/

#ifndef _STL_PTR_H
#define _STL_PTR_H

#include <assert.h>
#include <iostream.h>

template<class X> class stl_ptr
{
protected:
  X *ptr;
  mutable int* count;

  X* create(X* p=0)
    {
      ptr=p;
      if (ptr)
        count=new int(1);
      else count=0;
      return p;
    }
  void destroy()
    {
      if (ptr)
        if (--(*count)==0)
          {
            delete ptr;
            delete count;
          }
    }
  void copy(const stl_ptr<X>& a)
    {
      ptr=a.ptr;
      if (ptr)
        {
          count=a.count;
          (*count)++;
        }
      else count=0;
    }
public:
  explicit stl_ptr(X* p=0)            { create(p); }
  stl_ptr(const stl_ptr<X>& a)        { copy(a); }
  ~stl_ptr()                          { destroy(); }
  X* reset(X* p=0)                    { destroy(); return create(p); }
  void operator=(const stl_ptr<X>& a) { if (&a != this) { destroy();
copy(a); } }
  X& operator*() const                { assert(ptr); return *ptr; }
  X* operator->() const               { assert(ptr); return ptr; }
  X* get() const                      { return ptr; }
};

// operator == and operator < required for list container

template <class X>
bool operator==(stl_ptr<X> x, stl_ptr<X> y) { return x.get()==y.get(); }

template <class X>
bool operator<(stl_ptr<X> x, stl_ptr<X> y) { return x.get()<y.get(); }

#endif
