#include "cubic.h"

ScalarType ErrorOfCubic(ScalarType val, ScalarType deriv, ScalarType radius)
  {
    assert(val>=0);
    assert(deriv>=0);
        // let f(0)=val, f'(0)=deriv, f(radius)=0, f'(radius)=0
        // what is the maximum value of f?  What are the
        // roots of f'?  It has one root at radius (which we wish
        // to ignore).
        // f(x) = (val*(ax+b)+deriv*(cx+d))*(x/radius-1)^2
        // f(0) = (b*val+d*deriv) => b=1 and d=0
        // f(x) = (val*(ax+1)+deriv*(cx))*(x/radius-1)^2
        // f'(0) = (val*a+deriv*c) + val*(-1)*2/radius
        // so c=1 and a=2/radius
        // f(x) = (val*((2x/radius)+1)+deriv*x)*(x/radius-1)^2
        // f'(x) = (x/radius-1)*((val*2/radius+deriv)(x/radius-1)+
        //                       (val*((2x/radius)+1)+deriv*x)*(2/radius))
        // factoring out (x/radius-1) and setting = 0
        // ((val*2/radius+deriv)/radius+val*(2/radius)^2+deriv*2/radius)x
        //  = (val*2/radius+deriv)-val*2/radius = deriv
        // foo * x / radius^2 = deriv
        // where foo = val*2 + deriv*radius + val*4 + deriv*2*radius
        //           = val*6 + 3*deriv*radius

    if (val+deriv>0)
      {
        ScalarType x = deriv*(radius*radius)/(6*val + 3*radius*deriv);
        // plugging that x into f(x) gives the max
        ScalarType xoverr = x / radius;
        assert(xoverr<1);
        // f(x) = (val*(2*xoverr+1)+deriv*x)*(xoverr-1)^2
        ScalarType f=xoverr-1;
        f *= f;
        f *= val*(2*xoverr+1)+deriv*x;
        return f;
      }
    else
      return 0;
  }
