#include "draw.h"
#include "jfstr.h"

int gDebugOutputFileCount=0;

void WriteEPSFile(const PS::Drawing &drawing, const char *filename, 
                 const string &creator, const string &title)
  {
    string towrite;
    drawing.WriteEPS(towrite, creator, title);
    ofstream file(filename);
    file << towrite;
    file.close();
  }

double ScaleUp(double x)
  {
    return 72+(72*4)*x;
  }

double Radius(double x)
  {
    return (72*4)*x;
  }

typedef std::list<Root2D::Contour> ContourList;
typedef ContourList::const_iterator ContourIter;

void DrawSplineContour(const std::list<Root2D::Contour> &results,
                       PS::Drawing &drawing, const PS::State &state)
  {
    for (ContourIter cur=results.begin(); cur!=results.end(); ++cur)
      {
        PS::Path path;
        assert(((*cur).mControl.size() % 3) == 1);
        path.MoveTo(ScaleUp((*cur).mControl[0](0,0)), 
                    ScaleUp((*cur).mControl[0](0,1)));
        for (int i=1; i<(*cur).mControl.size(); i+=3)
          {
            double pts[6];
            for (int j=0; j<3; ++j)
              {
                pts[2*j]  =ScaleUp((*cur).mControl[i+j](0,0));
                pts[2*j+1]=ScaleUp((*cur).mControl[i+j](0,1));
              }
            path.CurveTo(pts);
          }
        if ((*cur).mEndingSides[0]==-1)
          path.Close();
        drawing.StrokePath(path, state);
      }
  }

void DrawLinearContour(const std::list<Root2D::Contour> &results,
                       PS::Drawing &drawing, const PS::State &state)
  {
    for (ContourIter cur=results.begin(); cur!=results.end(); ++cur)
      {
        PS::Path path;
        path.MoveTo(ScaleUp((*cur).mControl[0](0,0)), 
                    ScaleUp((*cur).mControl[0](0,1)));
        for (int i=1; i<(*cur).mControl.size(); ++i)
          path.LineTo(ScaleUp((*cur).mControl[i](0,0)),
                      ScaleUp((*cur).mControl[i](0,1)));
        drawing.StrokePath(path, state);
      }
  }

void DrawVertices(const std::list<Root2D::Contour> &results,
                  ScalarType r, PS::Drawing &drawing, 
                  const PS::State &state)
  {
    for (ContourIter cur=results.begin(); cur!=results.end(); ++cur)
      {
        PS::Path path;
        for (int i=0; i<(*cur).mControl.size(); ++i)
          path.Circle(ScaleUp((*cur).mControl[i](0,0)), 
                      ScaleUp((*cur).mControl[i](0,1)), r);
        drawing.FillPath(path, state);
      }
  }

