#include "int1dln.h"
#include "matrix.h"
#include "root1d.h"

ElementAction InterpolantLinear1D 
::AnalyzeElement(MeshElement_VirtualBase<VertexType> *element, 
                                 ScalarType radius, VertexType *center, 
                                 ScalarType epsilon)
  {
    int i;
    assert(element->NumCorners()==2);
    VertexType *v[2];

    for (i=0; i<2; ++i)
      {
        v[i]=& element->Corner(i);
        assert(v[i]->mEvaluated & VertexType::ValueEvaluated);
      }

    ScalarType error;
    error=(v[0]->mFunctionValue+v[1]->mFunctionValue)*.5;
    error=Matrix::Abs(center->mFunctionValue-error);
    error+=mThirdOrderError*radius*radius*radius;
    error*=mConservativeError;

    if (v[0]->mFunctionValue>0)
      {
        if (center->mFunctionValue-error>0 && 
            v[0]->mFunctionValue-error>0 &&
            v[1]->mFunctionValue-error>0)
          return DiscardElement;
      }
    else
      {
        if (center->mFunctionValue+error<0 && 
            v[0]->mFunctionValue+error<0 &&
            v[1]->mFunctionValue+error<0)
          return DiscardElement;
      }

    assert(Matrix::Abs(v[1]->mPos(0,0)-v[0]->mPos(0,0)-2*radius)<0.000001);
    assert(Matrix::Abs(v[1]->mPos(0,0)-center->mPos(0,0)-radius)<0.000001);
    assert(Matrix::Abs(center->mPos(0,0)-v[0]->mPos(0,0)-radius)<0.000001);
    ScalarType min_deriv=
                Matrix::Abs((v[1]->mFunctionValue - v[0]->mFunctionValue)/
                             (2*radius));
    min_deriv=Matrix::Min(min_deriv,
                Matrix::Abs((v[1]->mFunctionValue - center->mFunctionValue)/
                             radius));
    min_deriv=Matrix::Min(min_deriv,
                Matrix::Abs((center->mFunctionValue - v[0]->mFunctionValue)/
                             radius));
    if (error < epsilon*min_deriv)
      return ContourElement;
    return RefineElement;
  }

void InterpolantLinear1D 
::InterpolateContour(std::list<ContourType> &results, 
                     MeshElementList_VirtualBase<VertexType> *elt_lst)
  {
    MeshElement_VirtualBase<VertexType> *element;
    while ((element=elt_lst->NextElement())!=0)
      {
        assert(element->NumCorners()==2);
        ScalarType x[2], y[2];
        for (int i=0; i<2; ++i)
          {
            VertexType *v=&element->Corner(i);
            x[i]=v->mPos(0,0);
            y[i]=v->mFunctionValue;
            assert(v->mEvaluated == VertexType::BothEvaluated);
          }
        if (Root1D::SignOfProduct(y[0], y[1])!=Root1D::Positive)
          {
            ScalarType denom=y[0]-y[1];
            if (denom!=0)
              {
                ScalarType part=Matrix::Abs(y[0]/denom);
                Root1D::AddResult(results, x[0]+part*(x[1]-x[0]));
              }
          }
      }
  }
