/* int1dmd.h */
#ifndef __INTERPOLANT_CUBIC_1D_MAXD_H
#define __INTERPOLANT_CUBIC_1D_MAXD_H

#include "int1dbs.h"
#include "func1dmd.h"
#include <memory> // for auto_ptr

//#include "template.h"
//#include "vert.h"
//#include "smallmat.h"
//#include "matrix.h"
//#include "meshelt.h"
//#include "root1d.h"
//#include <assert.h>
//#include <math.h>

class InterpolantCubic1DMaxD 
: public InterpolantCubic1DBase<VertexInfoMaxD<DomainType1d, Extra_1D_Cubic> >
  {
public:
    typedef InterpolantCubic1DMaxD thistype;
    typedef FunctionCubic1DMaxD FunctionType;
    FunctionType* mFunction;

    void EvaluateValue(VertexType &vert)
      {
        assert(! (vert.mEvaluated & VertexType::ValueEvaluated));
        vert.mFunctionValue=mFunction->EvaluateValue(vert.mPos(0,0));
        vert.mEvaluated |= VertexType::ValueEvaluated;
      }

    void EvaluateInterpolationData(VertexType &vert)
      {
        assert(! (vert.mEvaluated & VertexType::InterpolationDataEvaluated));
        //assert(vert.mEvaluated & VertexType::ValueEvaluated);
        vert.mDerivative=mFunction->EvaluateDerivative(vert.mPos(0,0));
        vert.mEvaluated |= VertexType::InterpolationDataEvaluated;
      }

    void EvaluateElement(MeshElement_VirtualBase<VertexType> *element)
      {
        VertexType *v[2];
        assert(element->NumCorners()==2);
        for (int i=0; i<2; ++i)
          {
            v[i]=& element->Corner(i);
            if (v[i]->mEvaluated != VertexType::BothEvaluated)
              {
                assert(v[i]->mEvaluated & VertexType::ValueEvaluated);
                EvaluateInterpolationData(*v[i]);
              }
          }
      }

    ScalarType SafeRadius(VertexType &vert, ScalarType maxRadius)
      {
        assert(maxRadius>0);
        if (! (vert.mEvaluated & VertexType::ValueEvaluated))
          EvaluateValue(vert);
        ScalarType center=vert.mPos(0,0);
        ScalarType maxderiv = mFunction->MaxDerivative(center, maxRadius);
        return SafeRadiusHelper(vert.mFunctionValue, maxderiv, maxRadius);
      }

    bool StoppingCriteria(MeshElement_VirtualBase<VertexType> *element, 
                          ScalarType radius, ScalarType epsilon)
      {
        for (int i=0; i<2; ++i)
          {
            VertexType *v=& element->Corner(i);
            assert(v->mEvaluated & VertexType::ValueEvaluated);
            if (! (v->mEvaluated & VertexType::InterpolationDataEvaluated))
              EvaluateInterpolationData(*v);
          }

        ScalarType maxerr;
        if (mFunction->mBoundType)
          maxerr = C4MaxError(radius, mFunction->mBound);
        else
          maxerr = C1MaxError(radius, mFunction->mBound);
        return StoppingCriteriaHelper(maxerr, epsilon, element);
      }


    // need to define, in descendant:
    // void Evaluate(VertexType &);
    // ScalarType SafeRadius(const VertexType &, Scalar maxRadius);
protected:
    ScalarType SafeRadiusHelper(ScalarType value, ScalarType maxderiv, ScalarType maxRadius)
      {
        ScalarType maxsafe = fabs(value/maxderiv);
        return Matrix::Min(maxsafe,maxRadius);
      }

    ScalarType C4MaxError(ScalarType radius, ScalarType maxf4)
      {
        // compute radius^4
        radius *= radius;
        radius *= radius; 
        return radius * maxf4 * (1./24); // diameter^4/384
      }

    ScalarType C1MaxError(ScalarType radius, ScalarType maxf2)
      {
        // compute radius^2
        radius *= radius;
        return radius * maxf2 * .5; // 1/2
      }

  };

#endif
