#include "int2dbs.h"

namespace Interp2D
  {
void RenumberContours(std::list<Root2D::Contour> 
                  &contours, int mapsize, const int mappairs[][2])
  {
    typedef Root2D::Contour ContourType;
    typedef std::list<ContourType> ContourList;
    typedef ContourList::iterator  ContourIter;
    for (ContourIter cur=contours.begin(); cur!=contours.end(); ++cur)
      for (int end=0; end<2; ++end)
        for (int mapind=0; mapind<mapsize; ++mapind)
          if ((*cur).mEndingSides[end]==mappairs[mapind][0])
            {
              (*cur).mEndingSides[end]=mappairs[mapind][1];
              break;
            }
  }


inline ScalarType Dot(const Matrix::SmallMatrix<ScalarType, 1, 2> &v0, 
                      const Matrix::SmallMatrix<ScalarType, 1, 2> &v1)
  {
    return v0(0,0)*v1(0,0)+v0(0,1)*v1(0,1);
  }


// C0 interpolant with quadratic precision
void NineParamater(Vertex_2D_Cubic_Base *v[3], 
                  Root2D::TriangularCubic &cubic)
  {
    typedef DomainType2d DomainType;
    typedef Vertex_2D_Cubic_Base VertexType;

    cubic.mY[0]=v[0]->mFunctionValue;
    cubic.mY[3]=v[1]->mFunctionValue;
    cubic.mY[9]=v[2]->mFunctionValue;

    // need edge vectors to dot with gradients
    DomainType2d edges[3];
    int i;
    for (i=0; i<3; ++i)
      Matrix::Subtract(edges[i], cubic.mP[(i+1)%3], cubic.mP[i]);
    cubic.mY[1]=cubic.mY[0]-Dot(edges[0], v[0]->mGradient)*(1./3);
    cubic.mY[4]=cubic.mY[0]+Dot(edges[1], v[0]->mGradient)*(1./3);
    cubic.mY[2]=cubic.mY[3]-Dot(edges[0], v[1]->mGradient)*(1./3);
    cubic.mY[6]=cubic.mY[3]+Dot(edges[1], v[1]->mGradient)*(1./3);
    cubic.mY[8]=cubic.mY[9]-Dot(edges[1], v[2]->mGradient)*(1./3);
    cubic.mY[7]=cubic.mY[9]+Dot(edges[2], v[2]->mGradient)*(1./3);

    cubic.mY[5]=(cubic.mY[1]+cubic.mY[2]+cubic.mY[4]+cubic.mY[6]+cubic.mY[7]
                 +cubic.mY[8])*.25
               -(cubic.mY[0]+cubic.mY[3]+cubic.mY[9])*(1./6);
  }

/*
// computes cross derivative of side "1" to be the average of the 
// derivative in that direction at the two adjacent corners
void ContourCubicTriangle(std::list<Root2D::Contour> 
                          &results, DomainType2d verts[3], 
                          ScalarType value[3], 
                          DomainType2d grad[3],
                          ScalarType epsilon, ScalarType epsilonR)
  {
    Root2D::TriangularCubic  cubic;
    int i;
    for (i=0; i<3; ++i)
      cubic.mP[i]=verts[i];
    // figure out cubic.mY[i]
    // corners are easiest
    cubic.mY[0]=value[0];
    cubic.mY[3]=value[1];
    cubic.mY[9]=value[2];

    // need edge vectors to dot with gradients
    DomainType2d edges[3];
    for (i=0; i<3; ++i)
      Matrix::Subtract(edges[i], verts[(i+1)%3], verts[i]);
    cubic.mY[1]=value[0]+Dot(edges[0], grad[0])*(1./3);
    cubic.mY[4]=value[0]-Dot(edges[2], grad[0])*(1./3);
    cubic.mY[2]=value[1]-Dot(edges[0], grad[1])*(1./3);
    cubic.mY[6]=value[1]+Dot(edges[1], grad[1])*(1./3);
    cubic.mY[8]=value[2]-Dot(edges[1], grad[2])*(1./3);
    cubic.mY[7]=value[2]+Dot(edges[2], grad[2])*(1./3);

    DomainType2d center=verts[0];
    for (i=1; i<3; ++i)
      Matrix::Add(center, center, verts[i]);
    Matrix::Scale(center, 1./3);

    DomainType2d tocenter;
    Matrix::Subtract(tocenter, center, verts[1]);

    ScalarType projection = DotScale(tocenter, edge[1], 
                                      1/Dot(edge[1], edge[1]));
    assert(1./3<=projection);
    assert(projection<=2./3);

    DomainType2d remainder=tocenter;
    Matrix::AddScaled(remainder, -projection, edge[1]);
    projection = projection*3-1;
    Matrix::Add(tocenter, grad[1], grad[2]);

    cubic.mY[5]= cubic.mY[6]+(cubic.mY[8]-cubic.mY[6])*projection
                 +0.5*Dot(remainder, tocenter);

    Root2D::FindContour(cubic, epsilon, epsilonR, results);
  }
*/
// computes cross derivative of side "1" to be the average of the 
// derivative in that direction at the two adjacent corners
void ConstructCubicTriangle(Root2D::TriangularCubic &cubic, 
                            ScalarType value[2],
                            DomainType2d grad[2])
  {
    // figure out cubic.mY[i]
    // corners are easiest
    cubic.mY[3]=value[0];
    cubic.mY[9]=value[1];

    // need edge vectors to dot with gradients
    DomainType2d edges[3];
    int i;
    for (i=0; i<3; ++i)
      Matrix::Subtract(edges[i], cubic.mP[(i+1)%3], cubic.mP[i]);
    cubic.mY[2]=value[0]-Dot(edges[0], grad[0])*(1./3);
    cubic.mY[6]=value[0]+Dot(edges[1], grad[0])*(1./3);
    cubic.mY[8]=value[1]-Dot(edges[1], grad[1])*(1./3);
    cubic.mY[7]=value[1]+Dot(edges[2], grad[1])*(1./3);

    DomainType2d center=cubic.mP[0];
    for (i=1; i<3; ++i)
      Matrix::Add(center, center, cubic.mP[i]);
    Matrix::Scale(center, 1./3);

    DomainType2d tocenter;
    Matrix::Subtract(tocenter, center, cubic.mP[1]);

    ScalarType projection = Dot(tocenter, edges[1])/Dot(edges[1], edges[1]);

    DomainType2d remainder=tocenter;
    Matrix::AddScaled(remainder, -projection, edges[1]);
    projection = projection*3-1;
    Matrix::Add(tocenter, grad[0], grad[1]);

    cubic.mY[5]= cubic.mY[6]+(cubic.mY[8]-cubic.mY[6])*projection
                 +0.5*Dot(remainder, tocenter);
    // does not set cubic.mY[0, 1, 4]
  }

void CloughTocher(Vertex_2D_Cubic_Base *v[3], 
                  Root2D::TriangularCubic cubics[3],
                  ScalarType epsilonR)
  {
    typedef DomainType2d DomainType;
    typedef Vertex_2D_Cubic_Base VertexType;

    DomainType center=v[0]->mPos;
    int i;
    for (i=1; i<3; ++i)
      Matrix::Add(center, center, v[i]->mPos);
    Matrix::Scale(center, 1./3);

    ScalarType value[2];
    DomainType grad[2];
    int side;
    for (side=0; side<3; ++side)
      {
        cubics[side].mP[0]=center;
        for (i=0; i<2; ++i)
          {
            VertexType *cur=v[(side+i)%3];
            cubics[side].mP[i+1] = cur->mPos;
            value[i]             = cur->mFunctionValue;
            grad[i]              = cur->mGradient;
          }
        ConstructCubicTriangle(cubics[side], value, grad);
      }

    // figure out cubics[side].mY[1,4]
    for (side=0; side<3; ++side)
      {
        int otherside=(side+2)%3;
        assert(Matrix::Abs(cubics[side].mY[2]-cubics[otherside].mY[7])
                  <epsilonR);
        assert(Matrix::Abs(cubics[side].mY[3]-cubics[otherside].mY[9])
                  <epsilonR);
        // this may only be right for equalateral triangles!
        // but it might actually be correct in general, weird
        cubics[side].mY[1] = (cubics[side].mY[2]+cubics[side].mY[5]+
                              cubics[otherside].mY[5])*(1./3);
        cubics[otherside].mY[4] = cubics[side].mY[1];
      }

    // figure out cubics[side].mY[0]
    cubics[0].mY[0] = (cubics[0].mY[1]+cubics[0].mY[4]+cubics[1].mY[4])
                        *(1./3);
    cubics[1].mY[0] = cubics[0].mY[0];
    cubics[2].mY[0] = cubics[0].mY[0];
  }


void SibsonSplitSquare(Vertex_2D_Cubic_Base *v[4], 
                       Root2D::TriangularCubic cubics[4],
                       ScalarType epsilonR)
  {
    typedef DomainType2d DomainType;
    typedef Vertex_2D_Cubic_Base VertexType;

    DomainType center=v[0]->mPos;
    int i;
    for (i=1; i<4; ++i)
      Matrix::Add(center, center, v[i]->mPos);
    Matrix::Scale(center, 1./4);

    ScalarType value[2];
    DomainType grad[2];
    int side;
    for (side=0; side<4; ++side)
      {
        cubics[side].mP[0]=center;
        for (i=0; i<2; ++i)
          {
            VertexType *cur=v[(side+i)%4];
            cubics[side].mP[i+1] = cur->mPos;
            value[i]             = cur->mFunctionValue;
            grad[i]              = cur->mGradient;
          }
        ConstructCubicTriangle(cubics[side], value, grad);
      }

    // figure out cubics[side].mY[1,4]
    for (side=0; side<4; ++side)
      {
        int otherside=(side+3)%4;
        assert(Matrix::Abs(cubics[side].mY[2]-cubics[otherside].mY[7])
                  <epsilonR);
        assert(Matrix::Abs(cubics[side].mY[3]-cubics[otherside].mY[9])
                  <epsilonR);
        // note: fewer vertices than above (C-T/triangular) case
        cubics[side].mY[1] = (cubics[side].mY[5]+
                              cubics[otherside].mY[5])*.5;
        // this way, the four vertices are planar!
        cubics[otherside].mY[4] = cubics[side].mY[1];
      }

    // figure out cubics[side].mY[0]
    cubics[0].mY[0] = (cubics[0].mY[1]+cubics[0].mY[4]+
                       cubics[2].mY[1]+cubics[2].mY[4])*.25;
    cubics[1].mY[0] = cubics[0].mY[0];
    cubics[2].mY[0] = cubics[0].mY[0];
    cubics[3].mY[0] = cubics[0].mY[0];
  }

void CubicHull(const Root2D::TriangularCubic &cubic,
               ScalarType & mn, ScalarType & mx)
  {
    mn=cubic.mY[0];
    mx=cubic.mY[0];
    for (int i=1; i<10; ++i)
      if (cubic.mY[i]<mn)
        mn=cubic.mY[i];
      else if (cubic.mY[i]>mx)
        mx=cubic.mY[i];
  }


void QuadraticMinAbsAvg(const Root2D::TriangularQuadraticUnit 
                         &quadratic, ScalarType & minabs,
                        ScalarType & avgabs)
  {
    ScalarType mn=quadratic.mY[0], mx=mn;
    avgabs=mn;
    int i;
    for (i=1; i<6; ++i)
      {
        avgabs+=quadratic.mY[i];
        if (quadratic.mY[i]<mn)
          mn=quadratic.mY[i];
        else if (mx<quadratic.mY[i])
          mx=quadratic.mY[i];
      }
    avgabs /= 6;
    if (mn<0 && mx>0)
      {
        minabs = 0;
        ScalarType delta[3]={ avgabs-mn, mx-avgabs, mx-mn };
        // give mn..avg total weight delta[1]/delta[2]
        // average weight delta[1]/(delta[2]*delta[0])
        // and  avg..mx total weight delta[0]/delta[2]
        // average weight delta[0]/(delta[2]*delta[1])
        // total weight=1
        // could use a better model, but it would be pretty complicated
        // for example: could use a piecewise quadratic weight function
        // with f(min)=0, f(max)=0, C1 continuous and 
        // integral min..max x f(x) dx = avg * integral min..max f(x) dx
        // Have 1 (interesting) degree of freedom and 1 linear constraint
        // but pretty long expressions...
        if (avgabs>0)
          { // mn--0--avg--mx
            avgabs -= -mn*mn*delta[1]/(delta[0]*delta[2]);
          }
        else
          { // mn--avg--0--mx
            avgabs -= mx*mx*delta[0]/(delta[1]*delta[2]);
          }
      }
    else if (mx<=0)
      minabs = -mx;
    else
      minabs = mn;
    // avgabs may be <0 but it will be squared anyway...
  }

}