#ifndef __INTERPOLANT_2D_PRECISE_H
#define __INTERPOLANT_2D_PRECISE_H

#include "int2d.h"

struct PreciseInterpStrategy
  {
    typedef Interpolant2DBase<VertexInfo<DomainType2d, Extra_2D_Cubic>, 
                              PreciseInterpStrategy> PreciseBase;
    static void TriangleInterpolant(PreciseBase *,
                        Vertex_2D_Cubic_Base **v,
                        Root2D::TriangularCubic *cubics,
                        ScalarType epsilonR);
    static void SquareInterpolant(PreciseBase *,
                        Vertex_2D_Cubic_Base **v,
                        Root2D::TriangularCubic *cubics,
                        ScalarType epsilonR);
  };

class Interpolant2DPrecise 
: public Interpolant2DBase<VertexInfo<DomainType2d, Extra_2D_Cubic>, 
                              PreciseInterpStrategy>
  {
public:
    typedef FunctionCubicAccurate2D FunctionType;
    Interpolant2D::CubicErrorData mErr;

    FunctionType *mFunction;

    Interpolant2DPrecise() : mErr(.3, 2, 1) // reasonable defaults
      { }

    void Evaluate(VertexType &vert)
      {
        assert(! (vert.mEvaluated & VertexType::ValueEvaluated));
        assert(! (vert.mEvaluated & VertexType::InterpolationDataEvaluated));
        vert.mEvaluated |= VertexType::ValueEvaluated |
                           VertexType::InterpolationDataEvaluated;
      }

    void EvaluateCorner(VertexType &vert)
      {
        if (! (vert.mEvaluated & VertexType::ValueEvaluated))
          {
            mFunction->ValueAndGrad(vert.mPos(0,0), vert.mPos(0,1), 
                                    vert.mFunctionValue, vert.mGradient(0,0),
                                    vert.mGradient(0,1));
            vert.mEvaluated |= VertexType::ValueEvaluated |
                               VertexType::InterpolationDataEvaluated;
          }
        else if (! (vert.mEvaluated & VertexType::InterpolationDataEvaluated))
          {
            mFunction->Grad(vert.mPos(0,0), vert.mPos(0,1), 
                            vert.mGradient(0,0), vert.mGradient(0,1));
            vert.mEvaluated |= VertexType::InterpolationDataEvaluated;
          }
      }

    void EvaluateCenter(VertexType &vert)
      {
        if (! (vert.mEvaluated & VertexType::ValueEvaluated))
          {
            vert.mFunctionValue=mFunction->Value(vert.mPos(0,0), vert.mPos(0,1));
            vert.mEvaluated |= VertexType::ValueEvaluated;
          }
      }

    void EvaluateElement(MeshElement_VirtualBase<VertexType> *element)
      {
        int numcorners=element->NumCorners();
        for (int i=0; i<numcorners; ++i)
          EvaluateCorner(element->Corner(i));
      }

    ElementAction AnalyzeElement(MeshElement_VirtualBase<VertexType> *element, 
                                 ScalarType radius, VertexType *center, 
                                 ScalarType epsilon);
  };

#endif
