#ifndef __MESH_ELEMENT_H
#define __MESH_ELEMENT_H

#include "vert.h"
#include <assert.h>

typedef enum { RefineElement, DiscardElement, ContourElement } ElementAction;

template<class Vertex_T>
class MeshElement_VirtualBase
  {
public:
    typedef Vertex_T VertexType;

    virtual int NumCorners() = 0;
    virtual VertexType & Corner(int corner) = 0;
    virtual int Id() = 0;
    virtual int Neighbor(int) = 0;
  };

template<class Mesh_T>
class MeshElement : public MeshElement_VirtualBase<Mesh_T::VertexType>
  {
public:
    typedef Mesh_T MeshType;
    typedef typename MeshType::ElementType ElementType;
    MeshType *mMesh;
    ElementType mElement;

    int NumCorners()
      { 
        assert(mMesh);
        assert(mMesh->ElementValid(mElement));
        return mMesh->NumCorners(mElement);
      }
    VertexType & Corner(int corner)
      {
        assert(mMesh);
        assert(mMesh->ElementValid(mElement));
        assert(corner>=0 && corner<NumCorners());
        return mMesh->Corner(mElement, corner);
      }
    int Id()
      {
        assert(mMesh);
        assert(mMesh->ElementValid(mElement));
        return mElement;
      }
    int Neighbor(int which)
      {
        assert(mMesh);
        assert(mMesh->ElementValid(mElement));
        return mMesh->Neighbor(mElement, which);
      }
  };

template<class Vertex_T>
class MeshElementList_VirtualBase
  {
public:
    typedef Vertex_T VertexType;

    // returns 0 if done
    virtual MeshElement_VirtualBase<Vertex_T> *NextElement() = 0;
  };

template<class Mesh_T>
class MeshElementList : public MeshElementList_VirtualBase<Mesh_T::VertexType>
  {
public:
    typedef Mesh_T MeshType;
    typedef typename MeshType::ElementType ElementType;
    MeshElement<MeshType> mMeshElt;
    ElementType mElement;

    void Init(MeshType *mesh)
      {
        mMeshElt.mMesh=mesh;
      }
    MeshElement_VirtualBase<VertexType> *NextElement()
      {
        mMeshElt.mElement=mMeshElt.mMesh->PopResult();
        if (mMeshElt.mElement==mMeshElt.mMesh->InvalidElement())
          return 0;
        else
          return &mMeshElt;
      }
  };


#endif