#include "myfunc.h"
#include "matrix.h"
#include "smallmat.h"
#include "densemat.h"
// if you need QR-decomposition:
//#include "qr.h"
// and whatever includes you need
//#include <math.h>
#include <stdlib.h> // for rand()
#include <assert.h> // for assert()

using namespace Matrix;

const int cMatrixSize=9;

void MyFunction2D::Initialize(ScalarType minx, ScalarType maxx,
                              ScalarType miny, ScalarType maxy)
  {
    // do initialization and precomputation
    assert(maxx>minx);
    assert(maxy>miny);
    mMinX=minx;
    mMaxX=maxx;
    mMinY=miny;
    mMaxY=maxy;
    // 9x9 matrix with an extra row and column to avoid
    // worrying about accessing outside when doing bilinear interpolation
    mPrecomputedData.Initialize(cMatrixSize+1,cMatrixSize+1); 
    // random data in matrix
    for (int r=0; r<cMatrixSize+1; ++r)
      for (int c=0; c<cMatrixSize+1; ++c)
        mPrecomputedData(r,c)=(ScalarType(rand())/RAND_MAX)*2-1;
  }

void MyFunction2D
::Helper(ScalarType x, ScalarType y, int &c0, int &r0, 
         ScalarType &xr, ScalarType &yr)
  {
    ScalarType xn=(x-mMinX)*(cMatrixSize-1)/(mMaxX-mMinX);
    ScalarType yn=(y-mMinY)*(cMatrixSize-1)/(mMaxY-mMinY);
    c0=int(xn);
    r0=int(yn);
    xr=xn-c0;
    yr=yn-r0;
  }

// for linear interpolation case
ScalarType MyFunction2D
::Value(ScalarType x, ScalarType y)
  { // bilinear interpolation of mPrecomputedData
    int c0, r0;
    ScalarType xr, yr;
    Helper(x, y, c0, r0, xr, yr);
    // bilinear interpolation of data
    return mPrecomputedData(r0,c0)*(1-xr)*(1-yr)+
           mPrecomputedData(r0,c0+1)*xr*(1-yr)+
           mPrecomputedData(r0+1,c0)*(1-xr)*yr+
           mPrecomputedData(r0+1,c0+1)*xr*yr;
  }

// for cubic interpolation case, compute
// value, gradx, grady from x,y and precomputed data

void MyFunction2D
::ValueAndGrad(ScalarType x, ScalarType y, ScalarType &value, 
               ScalarType &gradx, ScalarType &grady)
  {
    int c0, r0;
    ScalarType xr, yr;
    Helper(x, y, c0, r0, xr, yr);
    // bilinear interpolation of data
    value=mPrecomputedData(r0,c0)*(1-xr)*(1-yr)+
          mPrecomputedData(r0,c0+1)*xr*(1-yr)+
          mPrecomputedData(r0+1,c0)*(1-xr)*yr+
          mPrecomputedData(r0+1,c0+1)*xr*yr;
    gradx=(mPrecomputedData(r0,c0)*(yr-1)+
           mPrecomputedData(r0,c0+1)*(1-yr)+
           mPrecomputedData(r0+1,c0)*(-yr)+
           mPrecomputedData(r0+1,c0+1)*yr)
          *(cMatrixSize-1);
    grady=(mPrecomputedData(r0,c0)*(xr-1)+
           mPrecomputedData(r0,c0+1)*(-xr)+
           mPrecomputedData(r0+1,c0)*(1-xr)+
           mPrecomputedData(r0+1,c0+1)*xr)
          *(cMatrixSize-1);
 }
