#ifndef MAKE_POSTSCRIPT_H
#define MAKE_POSTSCRIPT_H

#include "jstring.h"

namespace PS
  {
    class State
      {
    public:
        typedef enum { butt_cap, round_cap, square_cap } LineCapType;
        typedef enum { miter_join, round_join, bevel_join } JoinType;

    protected:
        friend class Drawing;
        friend class Path;
        double mLineThickness;
        string mColor;
        LineCapType mLineCap;
        JoinType mLineJoin;
        double mMiterLimit;

        State();
        void AppendDifference(const State &cmpto, string &result) const;

    public:
        // default operator= and copy constructor
        void SetLineThickness(double t)
          { mLineThickness = t; }
        void SetGray(double g);
        void SetRGB(double r, double g, double b);
        void SetLineCap(LineCapType c)
          { mLineCap=c; }
        void SetLineJoin(JoinType j)
          { mLineJoin=j; }
        void SetMiterLimit(double ml)
          { mMiterLimit=ml; }

        double GetLineThickness()
          { return mLineThickness; }
        LineCapType GetLineCap()
          { return mLineCap; }
        JoinType GetLineJoin()
          { return mLineJoin; }
        double GetMiterLimit()
          { return mMiterLimit; }
      };

    class Bounds
      {
    protected:
        friend class Drawing;
        double mBounds[4];
    public:
        Bounds();
        void Union(const Bounds &);
        void Enlarge(double);
        void Add(double, double);
        void Clear();
        bool Empty() const;
      };

    class Path
      {
    protected:
        string mCommands;
        Bounds mBounds;
        friend class Drawing;
        string GetCommands() const;
    public:
        // default operator=, copy copy constructor
        // default constructor
        void MoveTo(double, double);
        void LineTo(double, double);
        void CurveTo(double pts[6]);
        void Circle(double x, double y, double r);
        void Arc(double x, double y, double r, 
                 double degrees1, double degrees2);
        // text ?
        // ...
        void Close();
        void Outline(const State &state);
        void Clear();
      };

    class Drawing
      {
        string mCommands;
        Bounds mBounds;
        State mState;
        int pathnum;
    public:
        Drawing();

        State GetState()
          { return mState; }

        Path SavePath(const Path &path);

        void StrokePath(const Path &path, const State &state);
        void FillPath(const Path &path, const State &state);
        void DrawText(const char *str, 
                      double x, double y,
                      double size, const char *font, 
                      const State &state); // state includes color

        void WritePS(string &result, const string &creator, 
                     const string &title) const;
        void WriteEPS(string &result, const string &creator, 
                      const string &title) const;
      };
  }
#endif