#include "puzzle.h"
#include "int2dbs.h"
#include "merge2d.h"
#include <boost/scoped_array.hpp>

void Puzzle :: AddPiece(ContourList &results, int id, int numcorners, 
                        int *neighbors, ContourList &toadd, 
                        DomainType2d *cornerlocs)
  {
    int renum[8][2];
    int tomerge[8];
    int numtomerge=0;
    PuzzleCornerIter prevcorners[4];
    int i;
    for (i=0; i<numcorners; ++i)
      {
        renum[i][0] = i;
        renum[i+numcorners][0] = i+numcorners;
        renum[i+numcorners][1] = -1;
      }
    assert(mPuzzle.find(id)==mPuzzle.end());
    int numnewsides=0;
    int newsides[4];
    for (i=0; i<numcorners; ++i)
      {
        PuzzleIter nbr=mPuzzle.find(neighbors[i]);
        if (nbr==mPuzzle.end())
          {
            renum[i][1]=mCurNumber;
            mPuzzle[id].push_back();
            PuzzleSide &side=mPuzzle[id].back();
            side.mSide=mCurNumber;
            side.mNeighbor=neighbors[i];
            ++mCurNumber;
            newsides[numnewsides]=i;
            ++numnewsides;
          }
        else
          {
            PuzzleSideIter side_i=(*nbr).second.begin();
            for (; side_i!=(*nbr).second.end(); ++side_i)
              {
                if ((*side_i).mNeighbor==id)
                  break;
              }
            assert(side_i!=(*nbr).second.end());

            renum[i][1]=(*side_i).mSide;
            tomerge[numtomerge]=renum[i][1];
            ++numtomerge;
            for (int j=0; j<2; ++j)
              {
                int k=numcorners+((i+j)%numcorners);
                int c=*(*side_i).mCorners[1-j];
                if (renum[k][1]==-1)
                  {
                    renum[k][1]=c;
                    prevcorners[k-numcorners]=(*side_i).mCorners[1-j];
                  }
                else
                  {
                    assert(j==0 || i==numcorners-1);
                    if (renum[k][1]==c)
                      { // corner finished
                        tomerge[numtomerge]=renum[k][1];
                        ++numtomerge;
                        // see if same numbered corner has a different
                        // iterator
                        assert(*prevcorners[k-numcorners]==c);
                        if (prevcorners[k-numcorners]!=
                            (*side_i).mCorners[1-j])
                          mPuzzleCorners.erase(prevcorners[k-numcorners]);
                        mPuzzleCorners.erase((*side_i).mCorners[1-j]);
                        prevcorners[k-numcorners]=mPuzzleCorners.end();
                      }
                    else
                      { // merge two components
                        int xlate[1][2];
                        xlate[0][0]=c;
                        xlate[0][1]=renum[k][1];
                        Interp2D::RenumberContours(results, 1, xlate);
                        *(*side_i).mCorners[1-j] = renum[k][1];
                      }
                  }
              }
            (*nbr).second.erase(side_i);
          }
      }
    for (i=0; i<numcorners; ++i)
      if (renum[i+numcorners][1]==-1)
        {
          renum[i+numcorners][1]=mCurNumber;
          mPuzzleCorners.push_back(mCurNumber);
          prevcorners[i] = --mPuzzleCorners.end();
          ++mCurNumber;
        }
    if (numnewsides)
      {
        PuzzleSideIter psi=mPuzzle[id].begin();
        for (i=0; i<numnewsides; ++i)
          {
            // set (*psi).mCorners[0,1] to
            // prevcorners[newsides[i]+0,1 % numcorners]
            // assert neither are mPuzzleCorners.end()
            for (int j=0; j<2; ++j)
              {
                int k=(newsides[i]+j) % numcorners;
                assert(prevcorners[k]!=mPuzzleCorners.end());
                (*psi).mCorners[j]=prevcorners[k];
              }
            assert(psi!=mPuzzle[id].end());
            ++psi;
          }
      }
    assert(numcorners<=4);
    Merge2D::CornerNbhd corners[4];
    for (i=0; i<numcorners; ++i)
      {
        corners[i].mLocation=cornerlocs[i];
        corners[i].mSide=renum[i+numcorners][1];
      }

    Interp2D::RenumberContours(toadd, 2*numcorners, renum);
    if (numtomerge)
      Merge2D::MergeContours(results, toadd, mEpsilon, numtomerge, tomerge,
                             numcorners, corners);
    else
      results.splice(results.end(), toadd);
  }


void Puzzle  ::
FinishPuzzle(ContourList &results)
  {
    // each entry in puzzle can have at most 4 sides
    // normally this is a large overestimate
    boost::scoped_array<int> boundarymerge(new int[mPuzzle.size()*4 + 
                                                   mPuzzleCorners.size()]);
    int nummerge=0;
    for (PuzzleIter pi=mPuzzle.begin(); pi!=mPuzzle.end(); ++pi)
      {
        for (PuzzleSideIter psi=(*pi).second.begin(); 
             psi!=(*pi).second.end(); ++psi)
          {
            boundarymerge[nummerge]=(*psi).mSide;
            ++nummerge;
          }
      }
    for (PuzzleCornerIter pci=mPuzzleCorners.begin(); 
         pci!=mPuzzleCorners.end(); ++pci)
      {
        boundarymerge[nummerge]=(*pci);
        ++nummerge;
      }
    Merge2D::MergeContours(results, mEpsilon, nummerge, boundarymerge.get());
  }
