#ifndef __SINXX_FUNCTION_H
#define __SINXX_FUNCTION_H

#include "func1dmd.h"
#include "matrix.h"

/*
  f(x) = sin(100x^2) / (10*x);
  f'(x) = 20*cos(100*x*x)-sin(100*x*x)/(10*x*x);
  special care is taken so f is defined at and around 0
*/

class Function1DProfile  : public FunctionCubic1DMaxD
  {
public:
    int mFCount;
    int mGCount;
    void ResetCounts()
      {
        mFCount=0;
        mGCount=0;
      }
  };

class SinXXFunction : public Function1DProfile
  {
public:
    SinXXFunction()
      {
        mBoundType=0;
        mBound=2000;
      }
    ScalarType EvaluateValue(ScalarType x)
      {
        ++mFCount;
        if (x>1.e-10 || x<-1.e-10)
          return sin(100*x*x) / (10*x);
        else
          return 10*x;
      }
    ScalarType EvaluateDerivative(ScalarType x)
      {
        ++mGCount;
        if (x>1.e-10 || x<-1.e-10)
          return 20*cos(100*x*x)-sin(100*x*x)/(10*x*x);
        else
          return 10;
      }
    ScalarType MaxDerivative(ScalarType center, ScalarType radius)
      {
        return 20;
      }
  };
#endif