#ifndef __SMALL_MATRIX_H
#define __SMALL_MATRIX_H

#include <assert.h>

namespace Matrix {

// Class SmallMatrix<N,H,W> ---------------------------------
// N is either float, double, Complex, etc.
// H and W are the dimensions of the matrix and must be >0
template <class N, int H, int W>
class SmallMatrix
  {
protected:    
    N mElts[H][W];

public:
    typedef N Number;

    N operator()(int r, int c) const
      { assert(r>=0 && r<H);
        assert(c>=0 && c<W);
        return mElts[r][c];
      }
    N & operator()(int r, int c)
      { assert(r>=0 && r<H);
        assert(c>=0 && c<W);
        return mElts[r][c];
      }

    typedef SmallMatrix<N, H, 1> ColumnType;
    void InitializeColumn(ColumnType &c) const
      { }

    typedef SmallMatrix<N, 1, W> RowType;
    void InitializeRow(RowType &r) const
      { }

    typedef SmallMatrix<N, W, H> TransposeType;
    void InitializeTranspose(TransposeType &t) const
      { }

    int Width() const { return W; }
    int Height() const { return H; }
  };


template <class N>
N Length(SmallMatrix<N,1,1> m1, SmallMatrix<N,1,1> m2)
  {
    return Abs(m2(0,0)-m1(0,0));
  }

}
#endif